@@ -1,7 +1,7 @@
 package Getopt::Panjang;
 
 our $DATE = '2015-09-15'; # DATE
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -259,23 +259,45 @@ sub get_options {
     }
 
   RETURN:
-    my $success =
-        !keys(%unknown_opts) &&
-        !keys(%ambiguous_opts) &&
-        !keys(%val_missing_opts) &&
-        !keys(%val_invalid_opts);
-    [$success ? 200 : 500,
-     $success ? "OK" : "Error in parsing",
-     undef, {
-         'func.remaining_argv' => \@remaining,
-         ('func.unknown_opts'     => \%unknown_opts    )
-             x (keys(%unknown_opts) ? 1:0),
-         ('func.ambiguous_opts'   => \%ambiguous_opts  )
-             x (keys(%ambiguous_opts) ? 1:0),
-         ('func.val_missing_opts' => \%val_missing_opts)
-             x (keys(%val_missing_opts) ? 1:0),
-         ('func.val_invalid_opts' => \%val_invalid_opts)
-             x (keys(%val_invalid_opts) ? 1:0),
+    my ($status, $msg);
+    if (!keys(%unknown_opts) && !keys(%ambiguous_opts) &&
+            !keys(%val_missing_opts) && !keys(%val_invalid_opts)) {
+        $status = 200;
+        $msg = "OK";
+    } else {
+        $status = 500;
+        my @errs;
+        if (keys %unknown_opts) {
+            push @errs, "Unknown option" .
+                (keys(%unknown_opts) > 1 ? "s ":" ") .
+                join(", ", map {"'$_'"} sort keys %unknown_opts);
+        }
+        for (sort keys %ambiguous_opts) {
+            push @errs, "Ambiguous option '$_' (" .
+                join("/", @{$ambiguous_opts{$_}}) . "?)";
+        }
+        if (keys %val_missing_opts) {
+            push @errs, "Missing required value for option" .
+                (keys(%val_missing_opts) > 1 ? "s ":" ") .
+                join(", ", map {"'$_'"} sort keys %val_missing_opts);
+        }
+        for (keys %val_invalid_opts) {
+            push @errs, "Invalid value for option '$_': " .
+                $val_invalid_opts{$_};
+        }
+        $msg = (@errs > 1 ? "Errors in parsing command-line options: " : "").
+            join("; ", @errs);
+    }
+    [$status, $msg, undef, {
+        'func.remaining_argv' => \@remaining,
+        ('func.unknown_opts'     => \%unknown_opts    )
+            x (keys(%unknown_opts) ? 1:0),
+        ('func.ambiguous_opts'   => \%ambiguous_opts  )
+            x (keys(%ambiguous_opts) ? 1:0),
+        ('func.val_missing_opts' => \%val_missing_opts)
+            x (keys(%val_missing_opts) ? 1:0),
+        ('func.val_invalid_opts' => \%val_invalid_opts)
+            x (keys(%val_invalid_opts) ? 1:0),
     }];
 }
 
@@ -294,7 +316,7 @@ Getopt::Panjang - Parse command-line options
 
 =head1 VERSION
 
-This document describes version 0.03 of Getopt::Panjang (from Perl distribution Getopt-Panjang), released on 2015-09-15.
+This document describes version 0.04 of Getopt::Panjang (from Perl distribution Getopt-Panjang), released on 2015-09-15.
 
 =head1 SYNOPSIS
 
@@ -315,23 +337,7 @@ This document describes version 0.03 of Getopt::Panjang (from Perl distribution
  if ($res->[0] == 200) {
      # do stuffs with parsed options, $opts
  } else {
-    warn "Can't parse command-line options";
-    if ($res->[3]{'func.unknown_opts'}) {
-        warn "There are some unknown options: ",
-            join(",", keys %{$res->[3]{'func.unknown_opts'}});
-    }
-    if ($res->[3]{'func.ambiguous_opts'}) {
-        warn "There are some ambiguous options: ",
-            join(",", keys %{$res->[3]{'func.ambiguous_opts'}});
-    }
-    if ($res->[3]{'func.val_missing_opts'}) {
-        warn "There are some options which miss values: ",
-            join(",", keys %{$res->[3]{'func.val_missing_opts'}});
-    }
-    if ($res->[3]{'func.val_invalid_opts'}) {
-        warn "There are some options with invalid values: ",
-            join(",", keys %{$res->[3]{'func.val_invalid_opts'}});
-    }
+     die $res->[1];
  }
 
 Sample success result when C<@ARGV> is C<< ["--baz", 1, "--bar"] >>:
@@ -342,7 +348,7 @@ Sample error result (ambiguous option) when C<@ARGV> is C<< ["--ba", 1] >>:
 
  [
    500,
-   "Error in parsing",
+   "Ambiguous option 'ba' (bar/baz?)",
    undef,
    {
      "func.ambiguous_opts" => { ba => ["bar", "baz"] },
@@ -355,7 +361,7 @@ Sample error result (option with missing value) when C<@ARGV> is C<< ["--bar",
 
 [
    500,
-   "Error in parsing",
+   "Missing required value for option 'baz'",
    undef,
    {
      "func.remaining_argv"   => [],
@@ -367,7 +373,7 @@ Sample error result (unknown option) when C<@ARGV> is C<< ["--foo", "--qux"] >>:
 
  [
     500,
-   "Error in parsing",
+   "Unknown options 'foo', 'qux'",
    undef,
    {
      "func.remaining_argv" => ["--foo", "--qux"],
@@ -380,7 +386,7 @@ when C<@ARGV> is C<< ["--err", 1] >>:
 
  [
    500,
-   "Error in parsing",
+   "Invalid value for option 'err': Invalid value for option 'err': Bzzt\n",
    undef,
    {
      "func.remaining_argv"   => [],
@@ -462,27 +468,27 @@ Remaining argv after parsing is returned in the result metadata.
 
 Sample startup overhead benchmark:
 
-                            Rate      run_gl     load_gl      load_gp       run_gp run_gl_less load_gl_less run_gl_evenless load_gl_evenless   perl
- run_gl             71.95+-0.3/s          --       -3.8%       -67.2%       -68.0%      -68.1%       -71.7%          -75.3%           -81.2% -89.2%
- load_gl           74.81+-0.33/s 3.97+-0.62%          --       -65.9%       -66.7%      -66.8%       -70.6%          -74.3%           -80.4% -88.8%
- load_gp            219.6+-0.4/s 205.2+-1.4% 193.5+-1.4%           --        -2.2%       -2.5%       -13.7%          -24.5%           -42.5% -67.0%
- run_gp           224.54+-0.34/s 212.1+-1.4% 200.1+-1.4%  2.25+-0.24%           --       -0.4%       -11.8%          -22.8%           -41.2% -66.2%
- run_gl_less        225.3+-2.4/s 213.2+-3.6% 201.2+-3.5%    2.6+-1.1%    0.4+-1.1%          --       -11.5%          -22.5%           -41.0% -66.1%
- load_gl_less         254.5+-1/s   253.7+-2%   240.2+-2% 15.89+-0.51% 13.34+-0.49%  12.9+-1.3%           --          -12.5%           -33.4% -61.7%
- run_gl_evenless    290.7+-1.2/s 304.1+-2.3% 288.6+-2.3%  32.4+-0.59% 29.49+-0.56%    29+-1.5% 14.25+-0.66%              --           -23.9% -56.3%
- load_gl_evenless     382+-1.2/s 430.9+-2.7% 410.6+-2.7% 73.94+-0.62% 70.12+-0.58%  69.5+-1.9%  50.1+-0.76%    31.38+-0.67%               -- -42.6%
- perl                 665.2+-3/s 824.5+-5.6% 789.2+-5.6%  202.9+-1.5%  196.3+-1.4% 195.2+-3.4%  161.4+-1.6%     128.8+-1.4%     74.16+-0.94%     --
+                            Rate     load_gl      run_gl     load_gp       run_gp run_gl_less load_gl_less load_gl_evenless run_gl_evenless   perl
+ load_gl           73.23+-0.35/s          --       -2.6%      -65.4%       -65.5%      -68.0%       -70.9%           -78.9%          -80.0% -88.7%
+ run_gl            75.22+-0.17/s 2.71+-0.55%          --      -64.4%       -64.6%      -67.2%       -70.1%           -78.3%          -79.4% -88.3%
+ load_gp            211.4+-2.2/s 188.7+-3.3% 181.1+-2.9%          --        -0.5%       -7.7%       -16.0%           -39.1%          -42.2% -67.2%
+ run_gp           212.44+-0.86/s 190.1+-1.8% 182.4+-1.3%   0.5+-1.1%           --       -7.2%       -15.6%           -38.8%          -41.9% -67.1%
+ run_gl_less          229+-1.3/s 212.7+-2.3% 204.5+-1.8%   8.3+-1.3%   7.8+-0.74%          --        -9.0%           -34.0%          -37.4% -64.5%
+ load_gl_less     251.76+-0.83/s   243.8+-2% 234.7+-1.4%  19.1+-1.3% 18.51+-0.62% 9.93+-0.71%           --           -27.5%          -31.2% -61.0%
+ load_gl_evenless   347.1+-3.5/s   374+-5.3% 361.5+-4.8%  64.2+-2.4%   63.4+-1.8%  51.6+-1.7%   37.9+-1.5%               --           -5.1% -46.2%
+ run_gl_evenless    365.7+-1.8/s 399.4+-3.4% 386.2+-2.6%      73+-2%   72.1+-1.1%  59.7+-1.2% 45.26+-0.85%        5.3+-1.2%              -- -43.4%
+ perl               645.6+-6.8/s    782+-10% 758.3+-9.3% 205.3+-4.5%  203.9+-3.4% 181.9+-3.4%  156.4+-2.8%         86+-2.7%      76.5+-2.1%     --
  
  Average times:
-   perl            :     1.5033ms
-   load_gl_evenless:     2.6178ms
-   run_gl_evenless :     3.4400ms
-   load_gl_less    :     3.9293ms
-   run_gl_less     :     4.4385ms
-   run_gp          :     4.4535ms
-   load_gp         :     4.5537ms
-   load_gl         :    13.3672ms
-   run_gl          :    13.8985ms
+   perl            :     1.5489ms
+   run_gl_evenless :     2.7345ms
+   load_gl_evenless:     2.8810ms
+   load_gl_less    :     3.9720ms
+   run_gl_less     :     4.3668ms
+   run_gp          :     4.7072ms
+   load_gp         :     4.7304ms
+   run_gl          :    13.2943ms
+   load_gl         :    13.6556ms
 
 =head1 FUNCTIONS
 
